/****************************************************************************
* (C) 2006 by BECK IPC GmbH
*
*  BECK IPC GmbH
*  Germany
*  
*  http://www.beck-ipc.com
*
* ---------------------------------------------------------------------------
* Module        : EXTDISK.C
* Function      : External Disk Interface API Library
* ---------------------------------------------------------------------------

$Header: EXTDISK.C, 2, 29.08.2006 13:00:55, Andre Pribil$

****************************************************************************/



/****************************************************************************/
//includes
/****************************************************************************/

#include <dos.h>
#include <config.h>
#include <common.h>
#include <EXTDISK.H>
#include "EXT_PRIV.H"

/*
 @doc DOC_INDEX
 ---------------------------------------------------------------
 External Disk Interface API Table of Contents
 ---------------------------------------------------------------
 @clibidx  <tab><cfont>#include <InBraces EXTDISK\.H><efont> |

 @libcat1  Driver  | ext01 | Extdisk_install_drive    | Install a drive B: |
 @libcat1  Driver  | ext02 | Extdisk_remove_drive     | Deinstall the drive B: |
 @libcat1  Driver  | ext05 | Extdisk_install_drive2   | Install a drive D: |
 @libcat1  Driver  | ext06 | Extdisk_remove_drive2    | Deinstall the drive D: |

 @libcat2  Status  | ext03 | Extdisk_install_callback | Install a notify callback |
 @libcat2  Status  | ext04 | Extdisk_remove_callback  | Deinstall the notify callback |
*/


/*************************************************************************/
//Install drive B:
/*************************************************************************/
#ifdef _ext01_
/*
 @doc DOC_ext01_
 ---------------------------------------------------------------
 Install drive B:
 ---------------------------------------------------------------
 @clib  Extdisk_install_drive | Install a drive B: |
        <cfont>int Extdisk_install_drive ( unsigned long totalsectors );<efont> |
 @parm  unsigned long | totalsectors | Total sector count of disk
 @rdesc = 0: success<nl>
        != 0: failed
 @comm  Prior to this call the application must provide a disk read/write
        function installed at interrupt 0xB1.<s2> This installed handler
        function should expect:<ul><cfont>
        <nl>
        AX <s5> 1 for write, 0 for read.<nl>
        BX,DX <s2> Sector number (BX is MSH of unsigned long)<nl>
        CX <s5> Number of sectors to read/write<nl>
        ES:DI <s2> Segment:Offset of memory area to read/write<ule><efont>
        <nl>
        The handler should return 0 in AX if OK.
 @Also  ext02.htm | Extdisk_remove_drive |
        | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../extdisk.htm#0xB00x01<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90

 @target_clib V2.04
*/
int Extdisk_install_drive (unsigned long totalsectors)
{
    _BX = (unsigned int) (totalsectors >> 16);
    _DX = (unsigned int) totalsectors;
    _AH = EXTDISK_INSTALL_DRIVE;
    asm { int EXTDISKINT  } ;
    return _AX;
}
#endif

/*************************************************************************/
//Deinstall drive B:
/*************************************************************************/
#ifdef _ext02_
/*
 @doc DOC_ext02_
 ---------------------------------------------------------------
 Deinstall drive B:
 ---------------------------------------------------------------
 @clib  Extdisk_remove_drive | Deinstall the drive B: |
        <cfont>int Extdisk_remove_drive ( void );<efont> |
 @parm  byte |  | -- none --
 @rdesc = 0: success<nl>
        != 0: failed
 @Also  ext01.htm | Extdisk_install_drive |
        | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../extdisk.htm#0xB00x02<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.00 | V1.00 | V1.00 | V0.90

 @target_clib V2.04
*/
int Extdisk_remove_drive (void)
{
    _AH = EXTDISK_REMOVE_DRIVE;
    asm { int EXTDISKINT  } ;
    return _AX;
}
#endif


/*************************************************************************/
//Install callback
/*************************************************************************/
#ifdef _ext03_
/*
 @doc DOC_ext03_
 ---------------------------------------------------------------
 Install callback
 ---------------------------------------------------------------
 @clib  Extdisk_install_callback | Install a notify callback function |
        <cfont>int Extdisk_install_callback ( EXTDISK_CALLBACK callback );<efont> |
 @parm  EXTDISK_CALLBACK | callback | Callback function<nl>
        <nl>
        The following events are possible:<nl><ul>
        0x01: Drive B: is installed<nl>
        0x02: Drive B: is deinstalled<nl>
        0x03: Drive D: is installed<nl>
        0x04: Drive D: is deinstalled<nl><ule>
 @rdesc = 0: success<nl>
        != 0: failed
 @comm  Drive B: and D: can be hotpluggable. The driver of the drive will call
        the "Deinstall external disk" function, if the media in the drive is
        removed. If a new disk is inserted the driver will call the
        "Install external disk" function.<nl>
        Every task has a current working directory. If this directory is set
        to some directory on the drive and the media in the drive is removed,
        the current working directory will automatically be changed to A:\
        for this task, due to the call to the "Deinstall external disk"
        function of the driver.<nl><nl>
        The task can use the "Install notifiy callback function" to install a
        callback function on the events "drive is installed" and
        "drive is deinstalled" and react in some way.<nl><nl>
        The maximum number of installable callbacks is 12.
 @Also  ext04.htm | Extdisk_remove_callback |
        | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../extdisk.htm#0xB00x03<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.20 | V1.20 | V1.20 | V1.07

 @target_clib V2.04
*/
int Extdisk_install_callback (EXTDISK_CALLBACK callback)
{
    _ES = FP_SEG(callback);
    _DI = FP_OFF(callback);
    _AH = EXTDISK_INSTALL_CALLBACK;
    asm { int EXTDISKINT  } ;
    return _AX;
}
#endif


/*************************************************************************/
//Deinstall callback
/*************************************************************************/
#ifdef _ext04_
/*
 @doc DOC_ext04_
 ---------------------------------------------------------------
 Deinstall callback
 ---------------------------------------------------------------
 @clib  Extdisk_remove_callback | Deinstall the notify callback function |
        <cfont>int Extdisk_remove_callback ( EXTDISK_CALLBACK callback );<efont> |
 @parm  EXTDISK_CALLBACK | callback | Callback function<nl>
        Must be the same pointer as used in
        <linkto>ext03.htm<linkmid>Install notify callback function<linktail>
 @rdesc = 0: success<nl>
        != 0: failed
 @Also  ext03.htm | Extdisk_install_callback |
        | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../extdisk.htm#0xB00x04<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    V1.20 | V1.20 | V1.20 | V1.05

 @target_clib V2.04
*/
int Extdisk_remove_callback (EXTDISK_CALLBACK callback)
{
    _ES = FP_SEG(callback);
    _DI = FP_OFF(callback);
    _AH = EXTDISK_REMOVE_CALLBACK;
    asm { int EXTDISKINT  } ;
    return _AX;
}
#endif

/*************************************************************************/
//Install drive D:
/*************************************************************************/
#ifdef _ext05_
/*
 @doc DOC_ext05_
 ---------------------------------------------------------------
 Install drive D:
 ---------------------------------------------------------------
 @clib  Extdisk_install_drive2 | Install a drive D: |
        <cfont>int Extdisk_install_drive2 ( unsigned long totalsectors );<efont> |
 @parm  unsigned long | totalsectors | Total sector count of disk
 @rdesc = 0: success<nl>
        != 0: failed
 @comm  Prior to this call the application must provide a disk read/write
        function installed at interrupt 0xB2.<s2> This installed handler
        function should expect:<ul><cfont>
        <nl>
        AX <s5> 1 for write, 0 for read.<nl>
        BX,DX <s2> Sector number (BX is MSH of unsigned long)<nl>
        CX <s5> Number of sectors to read/write<nl>
        ES:DI <s2> Segment:Offset of memory area to read/write<ule><efont>
        <nl>
        The handler should return 0 in AX if OK.
 @Also  ext06.htm | Extdisk_remove_drive2 |
        | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../extdisk.htm#0xB00x05<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    n/a   | n/a   | n/a   | V1.07

 @target_clib V2.06
*/
int Extdisk_install_drive2 (unsigned long totalsectors)
{
    _BX = (unsigned int) (totalsectors >> 16);
    _DX = (unsigned int) totalsectors;
    _AH = EXTDISK_INSTALL_DRIVE2;
    asm { int EXTDISKINT  } ;
    return _AX;
}
#endif

/*************************************************************************/
//Deinstall drive D:
/*************************************************************************/
#ifdef _ext06_
/*
 @doc DOC_ext06_
 ---------------------------------------------------------------
 Deinstall drive D:
 ---------------------------------------------------------------
 @clib  Extdisk_remove_drive2 | Deinstall the drive D: |
        <cfont>int Extdisk_remove_drive2 ( void );<efont> |
 @parm  byte |  | -- none --
 @rdesc = 0: success<nl>
        != 0: failed
 @Also  ext05.htm | Extdisk_install_drive2 |
        | | |
 @rtos  This library function invokes a RTOS software interrupt.
        Refer to this RTOS API function's
        <linkto>../../extdisk.htm#0xB00x06<linkmid>documentation<linktail>
        for more details.
 @target     SC12  | SC13  | SC11  | SC1x3
 @version    n/a   | n/a   | n/a   | V1.07

 @target_clib V2.06
*/
int Extdisk_remove_drive2 (void)
{
    _AH = EXTDISK_REMOVE_DRIVE2;
    asm { int EXTDISKINT  } ;
    return _AX;
}
#endif

// End EXTDISK.C ////////////////////////

